# -*- coding: utf-8 -*
#!/usr/bin/env python3

from .trigram_utils import asTuples
from .expressions import expressions
from .data import data

from operator import itemgetter
import re

# The maximum distance to add when a given trigram does
# not exist in a trigram dictionary. 
MAX_DIFFERENCE = 300

script = ''
numericData = {}

for script in data:
    languages = data[script] 
    name = ''
    numericData[script] = {}

    for name in languages:
        model = languages[name].split('|')
        trigrams = {}
        weight = len(model)
        while weight > 0:   
            weight -= 1        
            trigrams[model[weight]] = weight

        numericData[script][name] = trigrams     

def lang_detect(value, **kwargs):
    # Minimum sample length.
    MIN_LENGTH = 10
    # Maximum sample length.
    MAX_LENGTH = 2048
    # Set whitelist and blacklist as default.
    only, ignore = [], []
    
    if kwargs.get('minlength'): MIN_LENGTH = kwargs.get('minlength') 
    
    if kwargs.get('whitelist'): only = kwargs.get('whitelist') 
        
    if kwargs.get('blacklist'): ignore = kwargs.get('blacklist') 
    
    if not value or len(value) < MIN_LENGTH:
        return und()
  
    value = value[slice(0, MAX_LENGTH)]
    script = getTopScript(value, expressions)

    if not script[0] or not (script[0] in numericData):
        if not script[0] or script[1] == 0 or not allow(script[0], only, ignore):
            return und()

        return singleLanguageTuples(script[0])

    return normalize(value, getDistances(asTuples(value), numericData[script[0]], only, ignore))

def normalize(value, distances):
    min = distances[0][1]
    max = len(value) * MAX_DIFFERENCE - min
    index = 0

    while index < len(distances):           
        distances[index][1] = 1 - (distances[index][1] - min) / max  or 0
        index += 1
        
    return distances

def getTopScript(value, scripts):
    topCount = -1
    topScript = ''
    script = ''
    
    for script in scripts:
            count = getOccurrence(value, scripts[script])
            if count > topCount:
                topCount = count
                topScript = script
    
    return topScript, topCount

def getOccurrence(value, expression):
    count = re.findall(expression, value)

    return (len(count) if count else 0) / len(value) or 0

def getDistances(trigrams, languages, only, ignore): 
    languages = filterLanguages(languages, only, ignore) 
    distances = []
    language = ''

    if languages:
        for language in languages:
            distances.append([language, getDistance(trigrams, languages[language])])
      
    return und() if len(distances) == 0 else sorted(distances, key=itemgetter(1), reverse=False)

def getDistance(trigrams, model):
    distance = 0
    index = 0
    
    while index < len(trigrams):
        trigram = trigrams[index]
        difference = MAX_DIFFERENCE
        if trigram[0] in model:
            difference = trigram[1] - model[trigram[0]] - 1
            if difference < 0:
                difference = -difference
        index += 1        
                   
        distance += difference    

    return distance

def filterLanguages(languages, only, ignore):
    if len(only) == 0 and len(ignore) == 0:
        return languages

    filteredLanguages = {}
    language = ''

    for language in languages:
        if allow(language, only, ignore):
            filteredLanguages[language] = languages[language]

    return filteredLanguages

def allow(language, only, ignore):
    if len(only) == 0 and len(ignore) == 0:
        return True

    return (len(only) == 0 or language in only) and not language in ignore

def und():
    return singleLanguageTuples('und')

def singleLanguageTuples(language):
    return [[language, 1]]
 


# -*- coding: utf-8 -*
#!/usr/bin/env python3

import sys

def nGram(value, n):   
    if type(n) != int or n < 1 or n == sys.maxsize:
      return str(n) + ' is not a valid argument for n-gram.'
      
    nGrams = [] 
    source = ''

    if value == '':
      return nGrams

    source = value if value[slice(0)] else str(value)
    index = len(source) - n + 1

    if index < 1:
      return nGrams

    while index > 0:
      index -= 1    
      nGrams.append(source[slice(index, index + n)])
      
    return list(nGrams)



# -*- coding: utf-8 -*
#!/usr/bin/env python3

from argparse import ArgumentParser
from .franc import *

def main():
    parser = ArgumentParser()
    parser.add_argument("-s", "--string", 
                        dest="string", type=str, required=True,
                        help="Input string.")                         
                        
    parser.add_argument("-t", "--top",
                        dest="top", default=5, type=int, 
                        help="Print top results.")
                        
    parser.add_argument("-m", "--minlength",
                        dest="minlength", default=10, type=int, 
                        help="Minimum string length  to accept.")
                        
    parser.add_argument("-w", "--whitelist", nargs="*",  
                        dest="whitelist", default=[], 
                        help="Allow languages.")
                        
    parser.add_argument("-b", "--blacklist", nargs="*",
                        dest="blacklist", default=[], 
                        help="Disallow languages.")
                        
    parser.add_argument("-p", "--percentage",
                        dest="perc", action="store_true", default=False,
                        help="Print relative match value (in percent).")                           
                                 
    args = parser.parse_args()

    if args.string:
        result = lang_detect(value = args.string, minlength = args.minlength, whitelist = args.whitelist, blacklist = args.blacklist)
        for top in result[:args.top]:
            if not args.perc:
                print(top[0] + ' : ' + str(top[1]))
            else:
                print(top[0] + ' : ' + str(round(100 / sum(v for name, v in result[:args.top]) * top[1])) + '%')

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*
#!/usr/bin/env python3

from .n_gram import nGram

from operator import itemgetter
import re

def clean(value):
    if value is None:
        return ''

    # trim other symbols
    value = re.sub(r'[\u0021-\u0040]+', ' ', str(value))
    # equal trim in javascript
    value = re.sub(r'^\s+|\s+$', '', value)
    # to lowercase
    value = value.lower()
    #function collapseWhiteSpace(value): 
    value = re.sub(r'\s+', ' ', value)
    return value

def trigrams(value):
    return nGram(' ' + clean(value) + ' ', 3)

def asDictionary(value):
    values = trigrams(value)
    dictionary = {}
    index = 0

    while index < len(values):        
        if values[index] in dictionary:
            dictionary[values[index]] = dictionary[values[index]] + 1 #dictionary.get(values[index], 0) + 1
        else:
            dictionary[values[index]] = 1
        index += 1
          
    return dictionary

def asTuples(value):
    dictionary = asDictionary(value)
    tuples = []
    trigram = ''

    for trigram in dictionary:
        if trigram in dictionary:
            tuples.append([trigram, dictionary[trigram]])

    tuples = sorted(tuples, key=itemgetter(1), reverse=True)
    return tuples

def tuplesAsDictionary(tuples):
    dictionary = {}
    index = -1

    while index < len(tuples):
        dictionary[tuples[index][0]] = tuples[index][1]
        index += 1
        
    return dictionary



from setuptools import (
    setup,
    find_packages,
)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="pyfranc",
    version="0.1.1",
    description="Text language detection basic on trigrams.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cyb3rk0tik/pyfranc",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    entry_points={'console_scripts': ['pyfranc_cli = pyfranc.pyfranc_cli:main']},
    author="cyb3rk0tik",
    author_email="cyberkotik@riseup.net",
    python_requires=">=3",
    license='MIT',
)